@@ -1,7 +1,7 @@
 import json
 from types import MethodType
 from typing import Type, Set, Dict, Union, Generator, Callable, Any, Tuple
-
+from django.utils.datastructures import MultiValueDict
 from django.db.models import QuerySet
 from django.forms import (
     BaseForm,
@@ -80,6 +80,8 @@ class SiteformsMixin(BaseForm):
     is_submitted: bool = False
     """Whether this form is submitted and uses th submitted data."""
 
+    _cls_subform_field = SubformField
+
     Composer: Type['FormComposer'] = None
 
     def __init__(
@@ -186,11 +188,29 @@ class SiteformsMixin(BaseForm):
 
             # Use custom field for subforms.
             if subforms.get(field_name):
-                base_fields[field_name] = SubformField(
+                base_fields[field_name] = cls._cls_subform_field(
                     original_field=field,
                     validators=field.validators,
                 )
 
+    @classmethod
+    def _combine_dicts(cls, *, args: list, kwargs: dict, src: dict, arg_idx: int, kwargs_key: str) -> MultiValueDict:
+
+        try:
+            data_args = args[arg_idx]
+        except IndexError:
+            data_args = None
+
+        if data_args is None:
+            data_args = kwargs.pop(kwargs_key, {})
+
+        combined = MultiValueDict()
+        combined.update(src)
+        if data_args:
+            combined.update(data_args)
+
+        return combined
+
     def _preprocess_source_data(self, data: Union[dict, QueryDict]) -> Union[dict, QueryDict]:
         return data
 
@@ -200,6 +220,16 @@ class SiteformsMixin(BaseForm):
         src = self.src
         request = self.request
 
+        # Get initial data from instance properties (as per property_fields)
+        instance = kwargs.get('instance')
+        if instance:
+            property_fields = self._get_meta_option('property_fields', [])
+            if property_fields:
+                initial = {}
+                for property_field in property_fields:
+                    initial[property_field] = getattr(instance, property_field)
+                kwargs['initial'] = {**initial, **kwargs.get('initial', {})}
+
         # Handle user supplied data.
         if src and request:
             data = getattr(request, src)
@@ -209,10 +239,17 @@ class SiteformsMixin(BaseForm):
 
             if is_submitted and request.method == src:
 
+                data = self._combine_dicts(
+                    args=args, kwargs=kwargs,
+                    src=data, arg_idx=0, kwargs_key='data'
+                )
                 data = self._preprocess_source_data(data)
-
                 self.data = data
-                files = request.FILES
+
+                files = self._combine_dicts(
+                    args=args, kwargs=kwargs,
+                    src=request.FILES, arg_idx=1, kwargs_key='files'
+                )
                 self.files = files
 
                 if args:
@@ -434,8 +471,11 @@ class SiteformsMixin(BaseForm):
 
         return is_multipart
 
+    def _get_meta_option(self, name: str, default: Any) -> Any:
+        return getattr(getattr(self, 'Meta', None), name, default)
+
     def _get_widget_readonly_cls(self, field_name: str) -> Type[ReadOnlyWidget]:
-        return getattr(self.Meta, 'widgets_readonly', {}).get(field_name, ReadOnlyWidget)
+        return self._get_meta_option('widgets_readonly', {}).get(field_name, ReadOnlyWidget)
 
     def _clean_fields(self):
         # this ensures valid attributes on validation including that in formsets
@@ -570,6 +610,7 @@ class FilteringSiteformsMixin(SiteformsMixin):
 
         undef_choice_value = self.filtering_choice_undefined_value
 
+        # todo maybe support MultiValueDict
         # drop undefined values beforehand not to mess with them later
         data = {
             key: value
