@@ -1,9 +1,11 @@
+from datetime import date
+
 import pytest
-from django.forms import fields, ModelMultipleChoiceField
+from django.forms import ModelMultipleChoiceField
 
 from siteforms.composers.base import FormComposer, ALL_FIELDS
 from siteforms.tests.testapp.models import Thing, Another, Additional, AnotherThing, Link, WithThrough, ThroughModel
-from siteforms.toolbox import ModelForm, Form
+from siteforms.toolbox import ModelForm, Form, fields
 
 
 class Composer(FormComposer):
@@ -48,6 +50,14 @@ class MyForm(ModelForm):
         pass
 
 
+class MyFormWithProperty(MyForm):
+
+    aprop = fields.CharField(label='from property', required=False)
+
+    class Meta(MyForm.Meta):
+        property_fields = ['aprop']
+
+
 class LinkForm(ModelForm):
 
     subforms = {
@@ -105,13 +115,18 @@ def test_args_data(request_post):
     thing = Thing()
     thing.save()
 
-    form = MyForm({'fchar': '1'}, src='POST', request=request_post(data={
-        '__submit': 'siteform',
-        'fchar': '2',
-    }))
+    form = MyForm(
+        {'fchar': '1', 'ftext': 'populated'},
+        src='POST',
+        request=request_post(data={'__submit': 'siteform', 'fchar': '2', 'fchoices': '2'}),
+    )
 
-    # automatic `src` handling overrides `data` as the first arg
-    assert form.data['fchar'] == '2'
+    # both datas are combined, src-request data has lesser priority
+    # automatic `src` handling doesn't override `data` as the first arg or kwarg
+    data = form.data.dict()
+    assert data['fchar'] == '1'
+    assert data['ftext'] == 'populated'
+    assert data['fchoices'] == '2'
 
 
 def test_fields_disabled_all():
@@ -431,11 +446,12 @@ def form_cls(form):
 
             first = fields.CharField(label='some', help_text='some help')
             second = fields.ChoiceField(label='variants', choices={'1': 'one', '2': 'two'}.items())
+            th = fields.DateField(label='date', initial=date(2023, 1, 1), required=False)
 
         form_kwargs = dict(
             composer=MyComposer,
             somefield=fields.CharField(),
-            fchar=fields.CharField(max_length=30),
+            fchar=fields.CharField(max_length=100),
             subforms={'fchar': SubForm1},
             model=model,
             fields=use_fields,
@@ -476,25 +492,27 @@ def test_json_subforms(form_cls, request_get, request_post):
 
     # Value is too long for subform field.
     valid, frm = check_source(request_get(
-        'some?__submit=siteform&somefield=bc&fchar-second=2&fchar-first=valueistoolong'))
+        'some?__submit=siteform&somefield=bc&fchar-second=2&fchar-first=' + ('x' * 300)))
     assert not valid
-    assert 'Ensure this value has at most 30' in f'{frm}'
+    assert 'Ensure this value has at most 100' in f'{frm}'
 
     # All is well.
     valid, frm = check_source(request_get(
         'some?__submit=siteform&somefield=bc&fchar-second=2&fchar-first=op'))
     assert valid
     assert 'field is required' not in f'{frm}'
-    assert frm.cleaned_data == {'fchar': '{"first": "op", "second": "2"}', 'somefield': 'bc'}
+    assert frm.cleaned_data == {'fchar': '{"first": "op", "second": "2", "th": null}', 'somefield': 'bc'}
 
     # With instance.
-    thing = Thing(fchar='{"first": "dum", "second": "2"}', ftext='one')
+    thing = Thing(fchar='{"first": "dum", "second": "2", "th": "2023-01-02"}', ftext='one')
     thing.save()
 
     valid, frm = check_source(request_get('some'), instance=thing)
+    assert not valid
     frm_html = f'{frm}'
     assert 'value="dum"' in frm_html
     assert 'value="2" selected' in frm_html
+    assert 'value="2023-01-02"' in frm_html
 
     # With instance save.
     valid, frm = check_source(request_post(data={
@@ -503,6 +521,7 @@ def test_json_subforms(form_cls, request_get, request_post):
         'ftext': 'two',
         'fchar-second': '1',
         'fchar-first': 'hi',
+        'fchar-th': '2023-01-03',
     }), instance=thing)
 
     assert valid
@@ -510,7 +529,7 @@ def test_json_subforms(form_cls, request_get, request_post):
 
     # get instead of refresh to get brand new objects
     thing = Thing.objects.get(id=thing.id)
-    assert thing.fchar == '{"first": "hi", "second": "1"}'
+    assert thing.fchar == '{"first": "hi", "second": "1", "th": "2023-01-03"}'
     assert thing.ftext == 'two'
 
 
@@ -577,6 +596,15 @@ def test_multipart(request_get):
     assert form.is_multipart()  # has nested form with FileField
 
 
+def test_model_with_property():
+    mything = Thing(fchar='yes')
+    mything.save()
+
+    form = MyFormWithProperty(instance=mything)
+    html = f'{form}'
+    assert 'value="he-yes"' in html
+
+
 def test_render_form_tag(form_cls, request_get):
 
     def spawn(**kwargs):
