@@ -6,6 +6,11 @@ from siteforms.toolbox import ReadOnlyWidget
 
 def test_basic(form):
 
+    class MyFcharWidget(ReadOnlyWidget):
+
+        def format_value_hook(self, value: Any):
+            return super().format_value_hook(value) + 'xxx'
+
     class MyWidget(ReadOnlyWidget):
 
         template_name = 'mywidget.html'
@@ -23,7 +28,10 @@ def test_basic(form):
             'widgets': {
                 'ftext': MyWidget,
                 'fm2m': MyMultipleWidget,
-            }
+            },
+            'widgets_readonly': {
+                'fchar': MyFcharWidget,  # test custom readonly widget
+            },
         },
     )
     foreign = Another.objects.create(fsome='that')
@@ -31,7 +39,7 @@ def test_basic(form):
     form = form_cls(instance=thing)
 
     html = f'{form}'
-    assert 'one</div>' in html  # Simple readonly
+    assert 'onexxx</div>' in html  # Simple readonly
     assert 'that</div>' in html  # FK
     assert '&lt;unknown (q)&gt;</div>' in html  # Unknown in choices
     assert 'mywidgetdata' in html  # data from template
